/*
 * Link.java (10.12.2007)
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.uzh.ifi.sparqlml.mining.core;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Class for handling the link values of a given Link in a subgraph.
 * 
 * @author Andr Locher
 */
public class Link {

	private List<String[]> linkList;

	private String linkName;

	/**
	 * Creates a new Link container.
	 */
	public Link(String[] name) {
		linkList = new ArrayList<String[]>();
		linkName = name[1];
	}

	/**
	 * Add a link to this Link container.
	 * 
	 * @param lnk
	 */
	public void addLink(String[] lnk) {
		if (!contains(lnk)) {
			linkList.add(lnk);
		}
	}

	/**
	 * Returns the link data at a given position
	 * 
	 * @param index
	 * @return
	 */
	public String[] getLinkData(int index) {
		return linkList.get(index);
	}

	/**
	 * Returns the number of links.
	 * 
	 * @return
	 */
	public int numLinks() {
		return linkList.size();
	}

	/**
	 * Returns true if this Link already contains the linkdata.
	 * 
	 * @return
	 */
	public boolean contains(String[] lnk) {
		for (int i = 0; i < linkList.size(); i++) {
			if (Arrays.equals(linkList.get(i), lnk)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Returns the name of the Link.
	 * 
	 * @return
	 */
	public String getName() {
		return linkName;
	}

	public String toString() {
		String output = "";
		for (int i = 0; i < linkList.size(); i++) {
			String[] curr = linkList.get(i);
			output += curr[0] + " -> " + curr[1] + " -> " + curr[2] + "\n";
		}
		return output;
	}
}
