/*
 * GraphPattern.java (10.12.2007)
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.uzh.ifi.sparqlml.mining.core.pattern;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import ch.uzh.ifi.sparqlml.arq.create.query.QueryML;

import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.sparql.syntax.ElementWalker;

/**
 * Utilities to access the query pattern in the WHERE clause
 * 
 * @author Andr Locher
 * 
 */

public class GraphPattern {

	public QueryML query;

	private QueryVisitor visitor;

	public HashSet<String[]> linkList;

	public HashSet<String[]> attrList;

	public List<String[]> miningModelVars;

	public GraphPattern(QueryML query) {
		this.query = query;
		init();
	}

	public GraphPattern(QueryML query, List<String[]> modelFeatures) {
		this.query = query;
		miningModelVars = modelFeatures;
		init();
	}

	private void init() {
		visitor = new QueryVisitor();
		ElementWalker.walk(query.getQueryPattern(), visitor);
		setGraphLists();
	}

	public ArrayList getVarPattern() {
		return visitor.getTriples();
	}

	public Triple getVarTriple(int index) {
		return visitor.getTriples().get(index);
	}

	public int hasVarTriples() {
		return visitor.getTriples().size();
	}

	// not needed anymore?
	public Node getRootNodeVar() {
		return visitor.getTriples().get(0).getSubject();
	}

	// not needed anymore?
	public Iterator getResultVars() {
		return query.getResultVars().iterator();
	}

	public HashSet<String[]> getLinkList() {
		return linkList;
	}

	public HashSet<String[]> getAttrList() {
		return attrList;
	}

	public boolean containsLinkItem(String[] check) {
		for (Iterator i = linkList.iterator(); i.hasNext();) {
			String[] linkItem = (String[]) i.next();
			if (Arrays.equals(check, linkItem))
				return true;
		}
		return false;
	}

	/**
	 * Detects the sets of links and attributes in the basic graph pattern.
	 */
	public void setGraphLists() {
		if (miningModelVars == null)
			miningModelVars = query.getMiningModelVars();

		linkList = new HashSet<String[]>();
		attrList = new HashSet<String[]>();

		for (int i = 0; i < miningModelVars.size(); i++) {
			String[] currVar = miningModelVars.get(i);

			if (isResource(currVar)) {
				for (int j = 0; j < hasVarTriples(); j++) {
					Triple t = getVarTriple(j);

					if (!t.getPredicate().getURI().startsWith(
							"java:ch.uzh.ifi.sparqlml.mining.")) {
						// get link to object
						// TODO: improve property function support
						if (t.getSubject().getName().equals(currVar[0])) {
							if (isMiningModelRes(t.getObject().getName())) {
								String[] linkItem = { currVar[0],
										t.getPredicate().getLocalName(),
										t.getObject().getName() };
								if (!containsLinkItem(linkItem))
									linkList.add(linkItem);
							}

							// get link from subject
						} else if (t.getObject().getName().equals(currVar[0])) {
							if (isMiningModelRes(t.getSubject().getName())) {
								String[] linkItem = { t.getSubject().getName(),
										t.getPredicate().getLocalName(),
										currVar[0] };
								if (!containsLinkItem(linkItem))
									linkList.add(linkItem);
							}
						}
					}
				}
			} else {
				// literal var
				for (int j = 0; j < hasVarTriples(); j++) {
					Triple t = getVarTriple(j);

					if (!t.getPredicate().getURI().startsWith(
							"java:ch.uzh.ifi.sparqlml.mining.")) {
						// get object for subject
						if (t.getSubject().getName().equals(currVar[0])) {
							if (isMiningModelRes(t.getObject().getName())) {
								String[] attrItem = { t.getObject().getName(),
										t.getPredicate().getLocalName(),
										currVar[0] };
								attrList.add(attrItem);
							}

							// get subject for object
						} else if (t.getObject().getName().equals(currVar[0])) {
							if (isMiningModelRes(t.getSubject().getName())) {
								String[] attrItem = { t.getSubject().getName(),
										t.getPredicate().getLocalName(),
										currVar[0] };
								attrList.add(attrItem);
							}
						}
					}
				}
			}

		}
	}

	public boolean isMiningModelRes(String var) {
		for (int i = 0; i < miningModelVars.size(); i++)
			if (miningModelVars.get(i)[0].equals(var)
					&& miningModelVars.get(i)[1].equals("RESOURCE")) {
				return true;
			}
		return false;
	}

	public boolean isResource(String[] var) {
		if (var[1].equals("RESOURCE"))
			return true;
		return false;
	}

}
