/*
 * ProximityApp.java (10.12.2007)
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.uzh.ifi.sparqlml.mining.module.proximity;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import kdl.prox.db.Container;
import kdl.prox.db.DB;
import kdl.prox.gui2.ProxURL;
import kdl.prox.model2.common.predictions.Predictions;
import kdl.prox.model2.common.sources.AttributeSource;
import kdl.prox.model2.common.sources.Source;
import kdl.prox.model2.rbc.RBC;
import kdl.prox.model2.rpt.RPT;
import kdl.prox.model2.rpt.modules.learning.DefaultLearningModule;
import kdl.prox.model2.rpt.modules.significance.DefaultSignificanceModule;
import kdl.prox.model2.rpt.modules.stopping.DefaultStoppingModule;
import kdl.prox.util.Util;
import ch.uzh.ifi.sparqlml.mining.core.DataContainer;
import ch.uzh.ifi.sparqlml.mining.core.Feature;
import ch.uzh.ifi.sparqlml.mining.core.Resources;
import ch.uzh.ifi.sparqlml.mining.core.Subgraphs;
import ch.uzh.ifi.sparqlml.mining.module.MiningApp;
import ch.uzh.ifi.sparqlml.mining.module.MiningModel;

import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.sparql.core.Var;

/**
 * Class for using classifiers of Proximity
 * 
 * Proximity: http://kdl.cs.umass.edu/proximity/
 * 
 * @author Andr Locher
 */
public class ProximityApp implements MiningApp {

	private ResultSet results;

	private Subgraphs s;

	private MiningModel model;

	private static String monet;

	private DB proxDB;

	private Predictions predictions;

	public ProximityApp() {
	}

	public void init(MiningModel m) {
		model = m;
		// TODO: Remove log message output
		Util.initLog4J();
		// intialize proximity database connection
		Literal hasConnection = (Literal) m.getMiningAppObject("hasConnection");
		monet = hasConnection.getString();
		if (!DB.isOpen()) {
			DB.open(monet);
		}
		DB.clearDB();
		DB.initEmptyDB();
	}

	public void prepareModelData(ResultSet res, HashMap<Var, Var> mappings) {
		results = res;
		// parse the data and create subgraphs
		DataContainer md = new DataContainer(results, model, mappings);

		md.processResults();
		s = md.getSubgraphs();
		// needed for proximity export
		Resources o = md.getResources();
		// call ProximityXML to export data to XML
		ProximityXML exporter = new ProximityXML(s, model.getModelName(), o);
		exporter.write();
	}

	public void learnModel() {
		Resource alg = model.getAlgorithm();
		List params = model.getModelParams();
		List algParams = model.getAlgorithmParams();

		if (alg.getURI().equals("http://kdl.cs.umass.edu/proximity/rpt")) {
			// create classifier for algorithm

			int maxDepth = 0;
			int threshold = 0;
			double pVal = 0.0;

			if (algParams.size() > params.size()) {
				for (int i = 0; i < algParams.size(); i++) {
					String[] curr = (String[]) algParams.get(i);
					if (curr[0].equals("maxDepth"))
						maxDepth = Integer.valueOf(curr[1]);
					else if (curr[0].equals("numThresh"))
						threshold = Integer.valueOf(curr[1]);
					else if (curr[0].equals("pVal"))
						pVal = Double.valueOf(curr[1]);
				}
			}

			for (int i = 0; i < params.size(); i++) {
				String[] curr = (String[]) params.get(i);
				if (curr[0].equals("maxDepth"))
					maxDepth = Integer.valueOf(curr[1]);
				else if (curr[0].equals("numThresh"))
					threshold = Integer.valueOf(curr[1]);
				else if (curr[0].equals("pVal"))
					pVal = Double.valueOf(curr[1]);
			}

			// set parameters
			RPT rpt = new RPT();
			DefaultLearningModule learnModule = (DefaultLearningModule) rpt.learningModule;
			DefaultStoppingModule stopModule = (DefaultStoppingModule) learnModule.stoppingModule;
			DefaultSignificanceModule sigModule = (DefaultSignificanceModule) learnModule.significanceModule;
			stopModule.setMaxDepth(maxDepth);
			stopModule.setMinSubgraphs(threshold);
			sigModule.setpVal(pVal);

			rpt.learn(getContainer(), getClassLabel(), getBackgroundFeatures());

			// save model to disk
			String fileName = model.getModelName() + "_RPT" + ".xml";
			rpt.save(modelsPath + fileName);
			model.setMiningModelFile(fileName);
		} else if (alg.getURI().equals("http://kdl.cs.umass.edu/proximity/rbc")) {

			RBC rbc = new RBC();

			ArrayList<AttributeSource> inputItemAttrs = new ArrayList<AttributeSource>();
			List<Source> old = getBackgroundFeatures();
			for (int i = 0; i < old.size(); i++) {
				inputItemAttrs.add((AttributeSource) old.get(i));
			}
			rbc.learn(getContainer(), getClassLabel(), inputItemAttrs);

			// save model to disk
			String fileName = model.getModelName() + "_RBC" + ".xml";

			rbc.save(modelsPath + fileName);
			model.setMiningModelFile(fileName);
		}
		DB.close();
	}

	public void applyModel() {
		Resource alg = model.getAlgorithm();

		if (alg.getURI().equals("http://kdl.cs.umass.edu/proximity/rpt")) {
			RPT rpt = new RPT();
			rpt.load(modelsPath + model.getMiningModelFile());
			predictions = rpt.apply(getContainer());
			predictions.setTrueLabels(getContainer(), getClassLabel());

		} else if (alg.getURI().equals("http://kdl.cs.umass.edu/proximity/rbc")) {
			RBC rbc = new RBC();
			rbc.load(modelsPath + model.getMiningModelFile());
			predictions = rbc.apply(getContainer());
			predictions.setTrueLabels(getContainer(), getClassLabel());
		}
		DB.close();
	}

	public Object getClassPrediction(int subgID) {
		if (subgID != 0) {
			return predictions.getInferredClass(String.valueOf(subgID) + "@0");
		} else {
			return null;
		}
	}

	public Object getTrueClass(int subgID) {
		return predictions.getTrueClass(String.valueOf(subgID) + "@0");
	}

	public Object getClassProbability(int subgID) {
		return predictions.getSubgProbability(subgID + "@0",
				(String) getClassPrediction(subgID));
	}

	public Object getAccuracy() {
		return "ACC: " + (1 - predictions.getZeroOneLoss());
	}

	public Object getAUC(String classLabel) {
		return predictions.getAUC(classLabel);
	}

	public Object getROCPoints(String classLabel) {
		String rocPoints = "";
		double[][] points = predictions.genRocPoints(classLabel);
		for (int i = 0; i < points.length; i++) {
			rocPoints += points[i][0] + "\t" + points[i][1] + System.getProperty("line.separator");
		}
		return rocPoints;
	}

	public Object getSquaredError() {
		return "Averaged squared loss: " + predictions.getRMSE();
	}

	public HashMap getSubMap() {
		return s.getSubMap();
	}

	/* ----------------------------------------- */
	/* Different Helper functions for Proximity */
	/* ----------------------------------------- */

	private Container getContainer() {
		ProxURL proxURL = new ProxURL("cont:/containers/"
				+ model.getModelName());
		Container container = proxURL.getContainer(false);
		return container;
	}

	private AttributeSource getClassLabel() {
		Feature classFeature = getClassFeature();

		String item = classFeature.getParent();
		String attribute = classFeature.getName();

		if (classFeature.isResource()) {
			item = s.getFeature(0).getName();
		}

		AttributeSource classLabel = new AttributeSource(item, attribute);
		return classLabel;
	}

	private Feature getClassFeature() {
		for (int i = 0; i < s.numFeatures(); i++) {
			Feature curr = s.getFeature(i);
			if (curr.isPredict()) { // only class
				// feature
				return curr;
			}
		}
		return null;
	}

	private List<Source> getBackgroundFeatures() {
		ArrayList<Source> inputItemAttrs = new ArrayList<Source>();

		// Feature.enableFeature("kdl.prox3.model.features.ReferenceVectorFeature");
		// { varName, featureType, isPredict, rootVar }

		for (int i = 0; i < s.numFeatures(); i++) {
			Feature curr = s.getFeature(i);
			if (!curr.isPredict() && !curr.isRoot() && !curr.isResource()) {
				String item = curr.getParent();
				if (curr.isResource())
					item = curr.getName();

				AttributeSource attrSrc = new AttributeSource(item, curr
						.getName());

				if (curr.isContinuous())
					attrSrc.setIsContinuous(true);

				inputItemAttrs.add(attrSrc);
			}
		}

		// add link attributes
		for (int i = 0; i < s.numLinks(); i++) {
			String[] curr = s.getLink(i);

			if (!curr[2].equals(getClassFeature().getName())) {
				AttributeSource linkAttr = new AttributeSource("link_"
						+ curr[1], curr[2]);
				if (!inputItemAttrs.contains(linkAttr))
					inputItemAttrs.add(linkAttr);
			}
		}

		return inputItemAttrs;
	}

}
