/*
 * WekaARFF.java (10.12.2007)
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.uzh.ifi.sparqlml.mining.module.weka;

import ch.uzh.ifi.sparqlml.mining.core.Feature;
import ch.uzh.ifi.sparqlml.mining.core.Subgraph;
import ch.uzh.ifi.sparqlml.mining.core.Subgraphs;
import ch.uzh.ifi.sparqlml.mining.module.BaseWriter;

/**
 * Class that handles the export of subgraphs to Weka ARFF format.
 * 
 * Weka: http://www.cs.waikato.ac.nz/ml/weka/
 * 
 * @author Andr Locher
 */

public class WekaARFF extends BaseWriter {

	/** Constant for datatype string. */
	public static final int STRING = 11;

	/** Constant for datatype date. */
	public static final int DATE = 22;

	/** Constant for datatype integer. */
	public static final int INTEGER = 33;

	/** Constant for datatype float. */
	public static final int FLOAT = 44;

	/** Constant for datatype double. */
	public static final int DOUBLE = 55;

	public Subgraphs s;

	public WekaARFF(Subgraphs subgraphs, String model) {
		super(model, "arff");
		s = subgraphs;
	}

	public void write() {
		writeHeader();
		writeData();
		importToWeka();
	}

	private void importToWeka() {

	}

	private void writeHeader() {
		writer.print("@RELATION " + model + "\n\n");

		for (int i = 0; i < s.numFeatures(); i++) {
			// varName, featureType, datatype, parent varName
			Feature attr = s.getFeature(i);
			if (!attr.isResource()) {
				String datatype = "";
				if (attr.getDatatype() == STRING) {
					String[] nominals = s.getNomValues(attr.getName());
					if (nominals.length > 0) {
						datatype = "{";
						for (int z = 0; z < nominals.length; z++) {
							datatype += nominals[z];
							if (z != nominals.length - 1)
								datatype += ",";
						}
						datatype += "}";
					} else {
						datatype = "STRING";
					}
				} else if (attr.getDatatype() == INTEGER
						|| attr.getDatatype() == FLOAT) {
					datatype = "NUMERIC";
				} else if (attr.getDatatype() == DATE) {
					datatype = "DATE";
				}
				writer.print("@ATTRIBUTE " + attr.getName() + " " + datatype
						+ "\n");
			} else if (!attr.isRoot()){
				String datatype = "";
				String[] nominals = s.getNomValues(attr.getName());
				if (nominals != null) {
					if (nominals.length > 0) {
						datatype = "{";
						for (int z = 0; z < nominals.length; z++) {
							datatype += nominals[z];
							if (z != nominals.length - 1)
								datatype += ",";
						}
						datatype += "}";
					} else {
						datatype = "STRING";
					}
				} else {
					datatype = "STRING";
				}
				writer.print("@ATTRIBUTE " + attr.getName() + " " + datatype
						+ "\n");
			}
		}
	}

	private void writeData() {
		writer.print("\n@DATA\n");

		for (int z = 0; z < s.numSubgraphs(); z++) {

			Subgraph currSub = s.getSubgraph(z);

			for (int i = 0; i < s.numFeatures(); i++) {

				Feature attr = s.getFeature(i);
				if (!attr.isResource()) {

					Feature currFeat = currSub.getFeature(i);
					if (!currFeat.isMissing()) {
						// TODO: write aggregators?
						String[] value = currFeat.getFeatureData(0);
						if (value.length > 1)
							writer.print(value[1]);
						else
							writer.print(value[0]);
					} else {
						writer.print("?");
					}
					if (i != s.numFeatures() - 1)
						writer.print(",");
				} else if (!attr.isRoot()){
					Feature currFeat = currSub.getFeature(i);
					if (!currFeat.isMissing()) {
						// TODO: write aggregators?
						String[] value = currFeat.getFeatureData(0);
						if (value.length > 1)
							writer.print(value[1]);
						else
							writer.print(value[0]);
					} else {
						writer.print("?");
					}
					if (i != s.numFeatures() - 1)
						writer.print(",");
				}
			}

			writer.print("\n");

		}

		writer.close();
	}

}
