/*
 * WekaApp.java (10.12.2007)
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.uzh.ifi.sparqlml.mining.module.weka;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;

import kdl.prox.model2.common.predictions.Predictions;
import weka.classifiers.Classifier;
import weka.classifiers.Evaluation;
import weka.classifiers.evaluation.ThresholdCurve;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.Utils;
import ch.uzh.ifi.sparqlml.mining.core.DataContainer;
import ch.uzh.ifi.sparqlml.mining.core.Subgraphs;
import ch.uzh.ifi.sparqlml.mining.module.MiningApp;
import ch.uzh.ifi.sparqlml.mining.module.MiningModel;

import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.sparql.core.Var;

/**
 * Class for using classifiers of Weka
 * 
 * Weka: http://www.cs.waikato.ac.nz/ml/weka/
 * 
 * @author Andr Locher
 */
public class WekaApp implements MiningApp {

	private ResultSet results;

	private MiningModel model;

	private double[] predictions;

	private Subgraphs s;

	private Instances test;

	private Evaluation eval;

	public WekaApp() {
	}

	public void init(MiningModel m) {
		model = m;
	}

	public void prepareModelData(ResultSet res, HashMap<Var, Var> mappings) {
		results = res;

		// parse the data and create subgraphs
		DataContainer md = new DataContainer(results, model, mappings);
		md.processResults();
		s = md.getSubgraphs();

		// call WekaARFF to export data to ARFF
		WekaARFF exporter = new WekaARFF(s, model.getModelName());
		exporter.write();
	}

	public void learnModel() {
		String[] options = null;
		String optionString = "";
		Resource alg = model.getAlgorithm();
		String algClass = model.getAlgorithmClass();
		List params = model.getModelParams();

		// get parameters
		for (int i = 0; i < params.size(); i++) {
			String[] curr = (String[]) params.get(i);
			for (int j = 0; j < curr.length; j++) {
				if (j == 0)
					optionString += "-" + curr[j] + " ";
				else
					optionString += curr[j] + " ";
			}
		}

		try {
			options = Utils.splitOptions(optionString);
		} catch (Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		Instances train = null;
		try {
			train = new Instances(new BufferedReader(new FileReader(dataPath
					+ model.getModelName() + ".arff")));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		train.setClassIndex(train.numAttributes() - 1);

		Classifier cls = null;
		try {
			cls = Classifier.forName(algClass, options);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		try {
			cls.buildClassifier(train);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// serialize model
		ObjectOutputStream oos = null;
		try {
			oos = new ObjectOutputStream(new FileOutputStream(modelsPath
					+ model.getModelName() + ".model"));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			oos.writeObject(cls);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			oos.flush();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			oos.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		model.setMiningModelFile(model.getModelName() + ".model");
	}

	public void applyModel() {

		// TODO: Add parameters for classifier
		Resource alg = model.getAlgorithm();
		Classifier cls = null;

		ObjectInputStream ois = null;
		try {
			ois = new ObjectInputStream(new FileInputStream(modelsPath
					+ model.getModelName() + ".model"));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			cls = (Classifier) ois.readObject();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			ois.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		test = null;
		try {
			test = new Instances(new BufferedReader(new FileReader(dataPath
					+ model.getModelName() + ".arff")));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		test.setClassIndex(test.numAttributes() - 1);

		eval = null;
		try {
			eval = new Evaluation(test);
		} catch (Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		try {
			predictions = eval.evaluateModel(cls, test);
		} catch (Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

	}

	public Object getClassPrediction(int subgID) {
		if (subgID - 1 < 0)
			return null;
		if (test.classAttribute().isNumeric())
			return predictions[subgID - 1];
		else
			return test.classAttribute().value((int) predictions[subgID - 1]);
	}

	public Object getClassProbability(int subgID) {
		return null;
	}

	public Object getAccuracy() {
		if (test.classAttribute().isNumeric())
			return "MAE: " + eval.meanAbsoluteError(); // return mean absolute
		// error
		else
			return "ACC: " + eval.pctCorrect() / 100; // return accuracy
	}

	public Object getAUC(String classLabel) {
		ThresholdCurve tc = new ThresholdCurve();
		int classIndex = 0;
		Instances result = tc.getCurve(eval.predictions(), classIndex);
		return tc.getROCArea(result);
	}

	public Object getROCPoints(String classLabel) {
		String rocPoints = "";
		ThresholdCurve tc = new ThresholdCurve();
		int classIndex = 0;
		Instances result = tc.getCurve(eval.predictions(), classIndex);

		double[] fpr = result.attributeToDoubleArray(4);
		double[] tpr = result.attributeToDoubleArray(5);
		for (int i = 0; i < fpr.length; i++) {
			rocPoints += fpr[i] + "\t" + tpr[i]
					+ System.getProperty("line.separator");
		}
		return rocPoints;
	}

	public Object getTrueClass(int subgID) {
		return test.classAttribute().value(
				(int) test.instance(subgID - 1).classValue());
	}

	public Object getAbsoluteLoss() {
		// TODO Auto-generated method stub
		return null;
	}

	public Object getSquaredError() {
		// TODO Auto-generated method stub
		return null;
	}

	public HashMap getSubMap() {
		return s.getSubMap();
	}

	/* ----------------------------------------- */
	/* Different Helper functions for Weka */
	/* ----------------------------------------- */

}
