/*
 * predict.java (10.12.2007)
 * 
 * See LICENSE for more information about licensing and warranties.
 */
package ch.uzh.ifi.sparqlml.mining.pfunction;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import ch.uzh.ifi.sparqlml.mining.core.Feature;
import ch.uzh.ifi.sparqlml.mining.module.MiningApp;
import ch.uzh.ifi.sparqlml.mining.module.MiningModel;

import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.query.QueryBuildException;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.sparql.core.Var;
import com.hp.hpl.jena.sparql.engine.ExecutionContext;
import com.hp.hpl.jena.sparql.engine.QueryIterator;
import com.hp.hpl.jena.sparql.engine.ResultSetStream;
import com.hp.hpl.jena.sparql.engine.binding.Binding;
import com.hp.hpl.jena.sparql.engine.iterator.QueryIterPlainWrapper;
import com.hp.hpl.jena.sparql.pfunction.PropFuncArg;
import com.hp.hpl.jena.sparql.pfunction.PropertyFunction;
import com.hp.hpl.jena.util.FileManager;

/**
 * Property function used by SPARQL-ML to predict
 * 
 * @author Andr Locher
 */
public class predict implements PropertyFunction {

	public Model model;

	private boolean isList = false;

	private Var prediction;

	private Var probability;

	public static String pathMMM = System.getenv("ARQROOT")
			+ System.getProperty("file.separator") + "mining"
			+ System.getProperty("file.separator") + "meta_models.rdf";

	public static String pathRes = System.getenv("ARQROOT")
			+ System.getProperty("file.separator") + "mining"
			+ System.getProperty("file.separator") + "results"
			+ System.getProperty("file.separator");

	public void build(PropFuncArg argSubject, Node predicate,
			PropFuncArg argObject, ExecutionContext execCxt) {
		Model container = ModelFactory.createDefaultModel();
		InputStream in = null;
		in = FileManager.get().open(pathMMM);

		if (in != null) {
			container.read(in, "");
			Node modelNode = null;
			if (argObject.isList())
				modelNode = argObject.getArg(0);
			else
				modelNode = argObject.getArg();
			if (!container.containsResource(container.asRDFNode(modelNode))) {
				throw new QueryBuildException("Can't find model "
						+ argObject.getArg(0));
			}
		} else {
			throw new QueryBuildException("Can't find meta model file");
		}

		if (argSubject.isList()) {
			isList = true;
			prediction = Var.alloc(argSubject.getArg(0));
			probability = Var.alloc(argSubject.getArg(1));
		} else {
			prediction = Var.alloc(argSubject.getArg());
		}
	}

	public QueryIterator exec(QueryIterator input, PropFuncArg argSubject,
			Node predicate, PropFuncArg argObject, ExecutionContext execCxt) {

		// open mining meta model (mmm) container
		Model container = ModelFactory.createDefaultModel();
		InputStream in = null;
		in = FileManager.get().open(pathMMM);
		container.read(in, "");

		model = ModelFactory.createDefaultModel();
		Node modelNode = null;
		if (argObject.isList())
			modelNode = argObject.getArg(0);
		else
			modelNode = argObject.getArg();
		Resource modelRes = container.createResource(modelNode.getURI());
		initModel(container, modelRes);
		MiningModel m = new MiningModel(model);

		// get MiningApp
		String appClass = m.getMiningAppClass();
		MiningApp app = null;
		try {
			app = (MiningApp) Class.forName(appClass).newInstance();
		} catch (ClassNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		app.init(m);

		// create Iterator over result bindings
		List<Binding> allBindings = new ArrayList<Binding>();
		while (input.hasNext()) {
			Binding binding = (Binding) input.next();
			allBindings.add(binding);
		}

		// prepare model data for MiningApp
		List<Feature> modelFeatures = m.getModelFeatures();

		QueryIterator it = new QueryIterPlainWrapper(allBindings.iterator(),
				execCxt);
		ResultSetStream rStream = new ResultSetStream(modelFeatures, model, it);

		app.prepareModelData(rStream, null);

		// apply model on data
		app.applyModel();

		// write out predictions
		HashMap subGraphs = app.getSubMap();

		HashMap<Integer, String> visitedGraphs = new HashMap<Integer, String>();
		HashMap<String, HashMap<String, Integer>> confMatrix = new HashMap<String, HashMap<String, Integer>>();

		Iterator bind = allBindings.iterator();
		while (bind.hasNext()) {
			int subgID = 0;
			Binding binding = (Binding) bind.next();
			Iterator vars = binding.vars();
			while (vars.hasNext()) {
				Var x = (Var) vars.next();
				// get subgID if current var is the rootVar
				if (x.getName().equals(modelFeatures.get(0).getName())
						&& binding.get(x).isURI()) {
					subgID = (Integer) subGraphs.get(binding.get(x).getURI());
				}
			}

			// add the new binding to the object variable
			if (app.getClassPrediction(subgID) != null) {
				if (isList) {
					binding.add(prediction, Node.createLiteral(String
							.valueOf(app.getClassPrediction(subgID))));
					if (app.getClassProbability(subgID) != null)
						binding.add(probability, Node.createLiteral(String
								.valueOf(app.getClassProbability(subgID))));
				} else {
					binding.add(prediction, Node.createLiteral(String
							.valueOf(app.getClassPrediction(subgID))));
				}
			}

			// create confusion matrix
			if (subgID != 0 && !visitedGraphs.containsKey(subgID)) {
				visitedGraphs.put(subgID, "");
				if (confMatrix.containsKey(app.getTrueClass(subgID))) {
					HashMap currMatrix = confMatrix.get(app
							.getTrueClass(subgID));
					if (currMatrix.containsKey(app.getClassPrediction(subgID))) {
						Integer amount = (Integer) currMatrix.get(app
								.getClassPrediction(subgID));
						amount++;
						currMatrix.put(app.getClassPrediction(subgID), amount);
					} else {
						currMatrix.put(app.getClassPrediction(subgID),
								new Integer(1));
					}
				} else {
					HashMap<String, Integer> values = new HashMap<String, Integer>();
					values.put((String) app.getClassPrediction(subgID),
							new Integer(1));
					confMatrix.put((String) app.getTrueClass(subgID), values);
				}
			}
		}

		// Add new binding for the accuracy of the prediction
		/*
		 * if (app.getAccuracy() != null) { Binding accuracy = new BindingMap();
		 * accuracy.add(prediction, Node.createLiteral(String.valueOf(app
		 * .getAccuracy()))); allBindings.add(accuracy); }
		 */
		saveResults(m, app, visitedGraphs, confMatrix);

		return new QueryIterPlainWrapper(allBindings.iterator(), execCxt);
	}

	private void saveResults(MiningModel m, MiningApp app,
			HashMap<Integer, String> visitedGraphs,
			HashMap<String, HashMap<String, Integer>> confMatrix) {
		// save results to file
		File file = new File(pathRes + now() + "_" + m.getModelName() + ".txt");
		BufferedWriter bw = null;
		try {
			bw = new BufferedWriter(new FileWriter(file));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			bw.write("=== Results Summary ===");
			bw.newLine();
			bw.newLine();
			bw.write(String.valueOf(app.getAccuracy()));
			bw.newLine();
			bw.write("Total Number of Instances: " + visitedGraphs.size());
			bw.newLine();
			bw.newLine();
			bw.write("=== Confusion Matrix ===");
			bw.newLine();
			bw.newLine();
			bw.write("True Class = { Predicted Class }");
			bw.newLine();
			bw.newLine();
			Iterator matrixIter = (Iterator) confMatrix.entrySet().iterator();
			while (matrixIter.hasNext()) {
				Map.Entry mapentry = (Map.Entry) matrixIter.next();
				bw.write(mapentry.toString());
				bw.newLine();
			}

			bw.newLine();
			bw.newLine();
			bw.write("=== ROC/AUC ===");
			bw.newLine();
			bw.newLine();

			String[] labels = null;
			List<Feature> features = m.getModelFeatures();
			for (int i = 0; i < features.size(); i++) {
				Feature f = features.get(i);
				if (f.isPredict()) {
					labels = f.getValues();
					break;
				}
			}

			for (int i = 0; i < labels.length; i++) {
				bw.write("Class: " + labels[i]);
				bw.newLine();
				bw.write("-------------------------------");
				bw.newLine();
				bw.write("AUC: " + app.getAUC(labels[i]));
				bw.newLine();
				bw.write("ROC: ( X , Y )");
				bw.newLine();
				bw.write("" + app.getROCPoints(labels[i]));
				bw.newLine();
				bw.newLine();
				bw.newLine();
			}

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			bw.flush();
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void initModel(Model container, Resource modelRes) {
		Iterator stmts = container.listStatements(modelRes, null,
				(RDFNode) null);

		while (stmts.hasNext()) {
			Statement s = (Statement) stmts.next();
			if (s.getObject().isURIResource()) {
				initModel(container, container.createResource(s.getObject()
						.asNode().getURI()));
			} else if (s.getObject().isAnon()) {

				initModel(container, container.createResource(s.getObject()
						.asNode().getBlankNodeId()));
			}
			model.add(s);
		}
	}

	public static String now() {
		String DATE_FORMAT_NOW = "yyyy-MM-dd_HH-mm-ss";
		Calendar cal = Calendar.getInstance();
		SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_NOW);
		return sdf.format(cal.getTime());
	}

}
