package ch.uzh.ifi.sparqlml.test;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import ch.uzh.ifi.sparqlml.arq.create.query.QueryExecutionFactoryML;
import ch.uzh.ifi.sparqlml.arq.create.query.QueryExecutionML;
import ch.uzh.ifi.sparqlml.arq.create.query.QueryFactoryML;
import ch.uzh.ifi.sparqlml.arq.create.query.QueryML;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.rdf.model.InfModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.reasoner.Reasoner;
import com.hp.hpl.jena.reasoner.ReasonerRegistry;
import com.hp.hpl.jena.util.FileManager;

public class ApplyInferenceModel {
	
	private static OntModel ontModel = ModelFactory.createOntologyModel();
	//private static Model ontModel = ModelFactory.createDefaultModel();
	//OntModelSpec.OWL_MEM_TRANS_INF

	static InfModel infmodel;

	public static void main(String[] args) {

		Reasoner reasoner = ReasonerRegistry.getOWLReasoner();
		String inGraphFileName = null;
		String inQueryFileName = null;
		String qstr = null;

		if (args.length > 0) {
			for (int i = 0; i < args.length; i++) {
				if (args[i].equals("--query"))
					inQueryFileName = args[i + 1];
				else if (args[i].equals("--data"))
					inGraphFileName = args[i + 1];
				else if (args[i].equals("--help"))
					usage();
			}
		}

		// read the query file
		System.out.println("Reading the query...");
		if (inQueryFileName != null) {
			qstr = inQueryString(inQueryFileName);
		}

		// read the data file
		System.out.println("Reading the data...");
		if (inGraphFileName != null && qstr != null) {
			InputStream in = FileManager.get().open(inGraphFileName);

			if (in == null)
				throw new IllegalArgumentException("File: " + inGraphFileName
						+ " not found");

			ontModel.read(in, "");
		}

		if (inGraphFileName == null) {
			InputStream in = FileManager
					.get()
					.open(
							"test/swrc/publication2affiliation/aifb_november2006_publication.owl");
			ontModel.read(in, "");
			InputStream in2 = FileManager
					.get()
					.open(
							"test/swrc/publication2affiliation/aifb_november2006_data_publication.owl");
			ontModel.read(in2, "");
		}


		if (qstr != null) {
			// Create - Prepare - Execute the query
			System.out.println("Executing the query...");

			// QueryFactoryML creates Parser for SPARQL-ML
			QueryML query = QueryFactoryML.create(qstr);

			// QueryExecutionML holds new method execCreate()
			QueryExecutionML qe = QueryExecutionFactoryML.create(query,
					ontModel);

			ResultSet test = qe.execSelect();
			ResultSetFormatter.out(test);
		}

	}

	// Read the query from file
	private static String inQueryString(String inQueryFileName) {
		String line = "";
		BufferedReader br = null;
		String inQueryString = "";

		try {
			br = new BufferedReader(new InputStreamReader(new FileInputStream(
					inQueryFileName)));
		} catch (FileNotFoundException e) {
			System.out.println(e);
		}

		try {
			while ((line = br.readLine()) != null) {
				inQueryString = inQueryString + line + "\n";
			}
		} catch (IOException e) {
		}

		return inQueryString;
	}

	private static void usage() {
		System.out.println("ch.unizh.ifi.sparqlml.arq.sparqlml [options]");
		System.out.println("Options:");
		System.out.println("--query");
		System.out.println("--data");
		System.out.println("--help");

		System.exit(0);
	}
}
