package ch.uzh.ifi.sparqlml.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import ch.uzh.ifi.sparqlml.arq.create.query.QueryExecutionFactoryML;
import ch.uzh.ifi.sparqlml.arq.create.query.QueryExecutionML;
import ch.uzh.ifi.sparqlml.arq.create.query.QueryFactoryML;
import ch.uzh.ifi.sparqlml.arq.create.query.QueryML;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.rdf.model.InfModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.reasoner.Reasoner;
import com.hp.hpl.jena.reasoner.ReasonerRegistry;
import com.hp.hpl.jena.util.FileManager;

public class InduceMultipleModel {

	// private static OntModel ontModel =
	// ModelFactory.createOntologyModel(OntModelSpec.OWL_MEM_TRANS_INF);
	private static Model ontModel = ModelFactory.createDefaultModel();

	public static void main(String[] args) {

		String inGraphFileName = null;
		String inQueryFileName = null;
		String qstr = null;

		if (args.length > 0) {
			for (int i = 0; i < args.length; i++) {
				if (args[i].equals("--query"))
					inQueryFileName = args[i + 1];
				else if (args[i].equals("--data"))
					inGraphFileName = args[i + 1];
				else if (args[i].equals("--help"))
					usage();
			}
		}

		// read the query file
		System.out.println("Reading the query...");
		if (inQueryFileName != null) {
			qstr = inQueryString(inQueryFileName);
		}

		/*
		 * // get domains File dirs = new File("test/owls-tc2/domains");
		 * String[] children = dirs.list();
		 *  // get files for (int i = 0; i < children.length; i++) { File dir =
		 * new File("test/owls-tc2/domains/" + children[i]); String[] files =
		 * dir.list(); int numFiles = files.length; int trainSet = (numFiles /
		 * 3) * 2; String domain = children[i]; for (int j = trainSet+1; j <
		 * numFiles; j++) { String file = files[j]; InputStream in =
		 * FileManager.get().open( "test/owls-tc2/domains/" + domain + "/" +
		 * file); ontModel.read(in, "", "RDF-XML/ABBREV"); }
		 *  }
		 */

		// read multiple data files
		String[] files = { "org.eclipse.compare_rcs_attr.owl",
				"org.eclipse.pde.build_rcs_attr.owl",
				"org.eclipse.pde.ui_rcs_attr.owl",
				"org.eclipse.search_rcs_attr.owl",
				"org.eclipse.update.core_rcs_attr.owl",
				"org.eclipse.update.ui_rcs_attr.owl" };
		if (files != null && qstr != null) {
			for (int i = 0; i < files.length; i++) {
				System.out.println("(" + (i + 1) + "/" + files.length + ") "
						+ files[i]);
				InputStream in = FileManager.get().open(
						"test/evoont/" + files[i]);
				ontModel.read(in, "");
			}
		}

		// read the data file
		System.out.println("Reading the data...");
		if (inGraphFileName != null && qstr != null) {
			InputStream in = FileManager.get().open(inGraphFileName);

			if (in == null)
				throw new IllegalArgumentException("File: " + inGraphFileName
						+ " not found");

			ontModel.read(in, "");
		}

		if (qstr != null) {
			// Create - Prepare - Execute the query
			System.out.println("Executing the query...");

			// QueryFactoryML creates Parser for SPARQL-ML
			QueryML query = QueryFactoryML.create(qstr);

			// QueryExecutionML holds new method execCreate()
			QueryExecutionML qe = QueryExecutionFactoryML.create(query,
					ontModel);

			Model testModel = qe.execCreate();
			testModel.write(System.out, "RDF/XML-ABBREV", "");
			// ResultSet test = qe.execSelect();
			// ResultSetFormatter.out(test);
		}

	}

	// Read the query from file
	private static String inQueryString(String inQueryFileName) {
		String line = "";
		BufferedReader br = null;
		String inQueryString = "";

		try {
			br = new BufferedReader(new InputStreamReader(new FileInputStream(
					inQueryFileName)));
		} catch (FileNotFoundException e) {
			System.out.println(e);
		}

		try {
			while ((line = br.readLine()) != null) {
				inQueryString = inQueryString + line + "\n";
			}
		} catch (IOException e) {
		}

		return inQueryString;
	}

	private static void usage() {
		System.out.println("ch.unizh.ifi.sparqlml.arq.sparqlml [options]");
		System.out.println("Options:");
		System.out.println("--query");
		System.out.println("--data");
		System.out.println("--help");

		System.exit(0);
	}

}
