package ch.uzh.ifi.sparqlml.test.owlstc;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Random;

import ch.uzh.ifi.sparqlml.arq.create.query.QueryExecutionFactoryML;
import ch.uzh.ifi.sparqlml.arq.create.query.QueryExecutionML;
import ch.uzh.ifi.sparqlml.arq.create.query.QueryFactoryML;
import ch.uzh.ifi.sparqlml.arq.create.query.QueryML;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.util.FileManager;

public class CrossValidater {

	public static String qstrInduce = "test/owls-tc2/induce.rq";
	public static String qstrApply = "test/owls-tc2/apply.rq";

	private static OntModel ontModel = ModelFactory.createOntologyModel(OntModelSpec.OWL_MEM_TRANS_INF);
//	 private static Model ontModel = ModelFactory.createDefaultModel();

	public static void main(String[] args) {
		String qstr = "";

		// get domains
		File dirs = new File("test/owls-tc2/domains");
		String[] children = dirs.list();

		// do 10-fold cross validation
		for (int x = 0; x < 10; x++) {
			ontModel.removeAll();
			qstr = inQueryString(qstrInduce);
			ArrayList<String> trainFiles = new ArrayList<String>();

			// get next domain
			for (int i = 0; i < children.length; i++) {
				File dir = new File("test/owls-tc2/domains/" + children[i]);
				String[] files = dir.list();
				double numFiles = files.length;
				int trainSet = 0;
				trainSet = (int) ((numFiles / 10) * 9);
				String domain = children[i];

				Random generator = new Random();

				for (int j = 0; j < trainSet; j++) {
					for(;;) {
						String file = files[generator.nextInt((int) numFiles)];
						if (!trainFiles.contains(file)) {
							trainFiles.add(file);
							InputStream in = FileManager.get().open(
									"test/owls-tc2/domains/" + domain + "/" + file);
							ontModel.read(in, "");
							break;
						}
					}
					
				}
			}

			// induce model
			System.out.println((x + 1) + ": Inducing Model...");
			QueryML queryInduce = QueryFactoryML.create(qstr);
			QueryExecutionML qeInduce = QueryExecutionFactoryML.create(
					queryInduce, ontModel);
			qeInduce.execCreate();
			ontModel.removeAll();
			
			qstr = inQueryString(qstrApply);
			
			// get next domain
			for (int i = 0; i < children.length; i++) {
				File dir = new File("test/owls-tc2/domains/" + children[i]);
				String[] files = dir.list();
				double numFiles = files.length;
				String domain = children[i];

				for (int j = 0; j < numFiles; j++) {
					String file = files[j];
					if (!trainFiles.contains(file)) {
						InputStream in = FileManager.get().open(
								"test/owls-tc2/domains/" + domain + "/" + file);
						ontModel.read(in, "");
					}
				}
			}

			// apply model
			System.out.println((x + 1) + ": Applying Model...");
			QueryML queryApply = QueryFactoryML.create(qstr);
			QueryExecutionML qeApply = QueryExecutionFactoryML.create(
					queryApply, ontModel);
			qeApply.execSelect();
			
		}

	}

	// Read the query from file
	private static String inQueryString(String inQueryFileName) {
		String line = "";
		BufferedReader br = null;
		String inQueryString = "";

		try {
			br = new BufferedReader(new InputStreamReader(new FileInputStream(
					inQueryFileName)));
		} catch (FileNotFoundException e) {
			System.out.println(e);
		}

		try {
			while ((line = br.readLine()) != null) {
				inQueryString = inQueryString + line + "\n";
			}
		} catch (IOException e) {
		}

		return inQueryString;
	}

}
